<?php

require __DIR__.'/vendor/autoload.php';
require __DIR__.'/vendor/zeedhi/framework/bootstrap.php';
require __DIR__.'/vendor/teknisa/libraries/scripts/bootstrap.php';
require __DIR__.'/vendor/zeedhi/dataexporter/bootstrap.php';  

$instanceManager = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();

$db              = __DIR__.'/../../../backend/config/db.xml';
$general         = __DIR__.'/../../../backend/config/general.xml';
$environment     = __DIR__.'/config/environment.xml';
$parameters      = __DIR__.'/config/parameters.xml';
$parametersProd  = __DIR__.'/../../../backend/config/parameters.xml';
$report          = __DIR__.'/config/report.xml';
$zhulog          = __DIR__.'/vendor/zeedhi/zhulog/config/log.xml';
$preferences     = __DIR__.'/vendor/zeedhi/preferences/services.xml';
$preferencesConfig = __DIR__.'/config/preferences.xml';

if(!realpath($db)) {
    $db = __DIR__."/config/db.xml";
}

$routes = array();
$routes[] = __DIR__.'/vendor/teknisa/libraries/libRoutes.json';
$routes[] = __DIR__ . "/vendor/zeedhi/preferences/routes.json";
$routes[] = __DIR__.'/vendor/zeedhi/dataexporter/routes.json';

$mapping = file_get_contents(__DIR__.'/config/servicesMapping.json');
$mapping = json_decode($mapping, true);

foreach ($mapping as $path) {
    $ext = substr(strrchr($path,'.'),1);
    if($ext === 'xml')
        $instanceManager->loadFromFile(__DIR__.$path);
    elseif($ext === 'json')
        $routes[] = __DIR__.$path;
}

$instanceManager->loadFromFile($db);
$instanceManager->loadFromFile($general);
$instanceManager->loadFromFile($report);
$instanceManager->loadFromFile($parametersProd);
$instanceManager->loadFromFile($parameters);
$instanceManager->loadFromFile($environment);
$instanceManager->loadFromFile($preferences);
$instanceManager->loadFromFile($preferencesConfig);

$container = $instanceManager->getContainer();

if ($container->hasParameter('use_zhu_acl') && $container->getParameter('use_zhu_acl') == 'true'){
    $acl = __DIR__ . '/../../acl/backend/config/acl.xml';

    if(realpath($acl))
        $instanceManager->loadFromFile($acl);

    require __DIR__ . "/vendor/zeedhi/zhuaclbackend/service/bootstrap.php";
    $routes[] = __DIR__ . "/vendor/zeedhi/zhuaclbackend/config/routes.json";
}

if ($container->hasParameter('use_zhu_update') && $container->getParameter('use_zhu_update') == 'true'){
    $zhu = __DIR__ . '/../../acl/backend/config/zhuUpdate.xml';

    if(realpath($zhu))
        $instanceManager->loadFromFile($zhu);
}

$logParam = "use_zhu_log";
if ($container->hasParameter($logParam) && $container->getParameter($logParam)) {
        $zhuDateFormat = $container->getParameter('zhuLogDateFormat');
        $ignoredRoutes = $container->hasParameter('logIgnoredRoutes') ? $container->getParameter('logIgnoredRoutes') : [];

        require __DIR__ . "/vendor/zeedhi/zhulog/config/bootstrap.php";
        $routes[] = __DIR__."/vendor/zeedhi/zhulog/config/routes.json";

        $log = __DIR__.'/../../acl/backend/config/log.xml';
        if(realpath($log)){
                $instanceManager->loadFromFile($log);
        }

        $libLog = __DIR__.'/vendor/teknisa/libraries/zhuLog.xml';
        if(realpath($libLog)){
                $instanceManager->loadFromFile($libLog);
        }

        $container->setParameter('logIgnoredRoutes', $ignoredRoutes);
        $container->setParameter('zhuLogDateFormat', $zhuDateFormat);
}
$instanceManager->registerService('routeParser', new \Zeedhi\Framework\Routing\Parsers\JSONFormat($routes));

$instanceManager->compile();

use Doctrine\DBAL\Types\Type;

$entity_manager = $instanceManager->getService('entityManager');
\Setforms2\Shared\Registry::store("EntityManager", $entity_manager);
\Setforms2\Shared\Registry::store("EntityManagerConnection", $entity_manager->getConnection());