module.exports = function (grunt) {

    var backendModules = ['../../teknisa/est'];
    var directories = ['utilities', 'repositories', 'services', 'controllers'];
    var subModules = ['ajuste', 'estoqueCego', 'lancamentos', 'relatorios'];
    var libFiles = [
        'zh-grid-navigation/src/GridNavigationLibrary.js',
        'zh-interval-component/dist/zh-interval-component.js',
        
        'zh-grid-configuration/dist/zh-grid-configuration.js',
        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-view-preferences/dist/view-preferences.js',
        
        'zh-cache-bypass/src/CacheBypass.js',
        'zh-reports/dist/zhReports.js',
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zhuaclfrontend/assets/dist/main.js',
        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js'
    ];

    grunt.initConfig({
        concat: {
            js: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getJSPaths(),
                dest: './assets/js/es6.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: getCSSPaths(),
                dest: './assets/css/app.css'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: './assets/js/libs.js'
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                // the banner is inserted at the top of the output
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    './assets/js/app.min.js': getJSPaths()
                }
            }
        },
        jshint: {
            all: getJSPaths().concat(["GruntFile.js"]),
            options: {
                evil: true,
                varstmt: false, //TO-DO: enable var blocking
                esversion: 6
            }
        },
        watch: {
            dev: {
                files: getJSPaths().concat(getCSSPaths(), getLibPaths()),
                tasks: [ 'jshint', 'concat']
            },
            env: {
                files: './config/devEnvironment.json',
                tasks: ['buildEnvironment:development']
            },
            acl: {
                files: './assets/json/menu.json',
                tasks: ['buildAclStruct']
            }
        },
        cssmin: {
            dist: {
                src: getCSSPaths(),
                dest: './assets/css/app.min.css'
            }
        },
        babel: {
            options: {
            compact: false,
                sourceMap: true,
                presets: ['env']
            },
            dist: {
                files: {
                    'assets/js/app.js': 'assets/js/es6.js'
                }
            }
        },
        copy: {
            main:{
                files: [{
                    expand: true,
                    cwd: 'node_modules/babel-polyfill/dist',
                    src: 'polyfill.js',
                    dest: 'assets/js/'
                }]
            }
        },
        // NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
        zhIdGenerator: {								// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
            options: {									// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
                jsonPath: getJSONPaths(),				// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
                menuPath: ["./assets/json/menu.json"]	// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
            }											// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
        },												// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
        // NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
        buildEnvironment: {
            development: {
                configFile: "config/devEnvironment.json"
            },
            test: {
                configFile: "config/testEnvironment.json"
            },
            production: {
                configFile: "config/prodEnvironment.json"
            },
            srcFolders: ['./', './assets/json', '../backend/config', '../../teknisa/est/reports/config', '../../teknisa/est/reports']
        },
        buildAclStruct: {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        zhWords: {
            options: {
                serverUrl: "https://translate.teknisa.com/backend/index.php",
                organization: "20",
                project: "2297"
            }
        },
        "merge-json": {
            messages: {
                src: ['assets/json/messages/*.json', 'assets/modules/**/assets/json/messages/*.json'],
                dest: 'assets/json/messages.json'
            }
        }
    }); 


    function getOrderedPaths(directory) {
        var paths = [];

        paths.push('./assets/' + directory + '/**/*Utilit{y,ies}.js');
        paths.push('./assets/' + directory + '/**/*Repositor{y,ies}.js');
        paths.push('./assets/' + directory + '/**/*Factor{y,ies}.js');
        paths.push('./assets/' + directory + '/**/*Servic{e,es}.js');
        paths.push('./assets/' + directory + '/**/*Controlle{r,rs}.js');
        paths.push('./assets/' + directory + '/**/*.js');

        return paths;
    }

    function getJSPaths() {
        var paths = [];
        paths.push('./assets/modules/produto/templates/controllers/*');
        paths.push('./assets/templates/controllers/*.js');
        directories.forEach(function (directory) {
            paths = paths.concat(getOrderedPaths('js/' + directory));
            subModules.forEach(function (subModule) {
                paths = paths.concat(getOrderedPaths('modules/' + subModule + '/assets/js/' + directory));
            });
        });
        return paths;
    }

    function getCSSPaths() {
        var paths = [];
        paths.push('./assets/css/**/*.css');
        paths.push('!./assets/css/app.css');
        paths.push('./assets/templates/css/*.css');
        paths.push('./assets/modules/produto/templates/styles/*');
        paths.push('./bower_components/zh-customizations-service/dist/style/colors/styles-pebbian.css');
        paths.push('./bower_components/zh-select-autocomplete/dist/style/colors/styles-pebbian.css');
        paths.push('./bower_components/zh-preferences/dist/style/style.css');
        paths.push('./bower_components/zh-preferences/dist/style/colors/styles-pebbian.css');
        paths.push('./bower_components/zh-grid-configuration/dist/style/style.css');
        paths.push('./bower_components/zh-grid-configuration/dist/style/colors/styles-pebbian.css');
        paths.push('./bower_components/zh-reports/dist/style/colors/styles-pebbian.css');
        paths.push('./bower_components/zh-grid-navigation/src/style.css');
        
        paths.push('./bower_components/zh-grid-configuration/dist/style/style.css');
        paths.push('./bower_components/zh-grid-configuration/dist/style/colors/styles-pebbian.css');
        paths.push('./bower_components/zh-reports/dist/style/colors/styles-pebbian.css');
        
        subModules.forEach(function (subModule) {
            paths.push('./assets/modules/' + subModule + '/assets/css/**/*.css');
            paths.push('!./assets/modules/' + subModule + '/assets/css/app.css');
        });
        return paths;
    }

    function getLibPaths() {
        var paths = [];
        libFiles.forEach(function(fileName) {
            paths.push('bower_components/' + fileName);
        });
        return paths;
    }

    function getJSONPaths() {
        var paths = ["./assets/json"];
        subModules.forEach(function (subModule) {
            paths.push("./assets/modules/" + subModule + "/assets/json");
        });
        return paths;
    }
    
    function unicodeEscape(s) {
      return s.replace(/[^0-9|^a-z]/gi, (char) => { 
        return `\\u${(`000${char.charCodeAt(0).toString(16).toUpperCase()}`).substr(-4)}`;
      });
    }

    function buildFileContent(srcFiles, filePath, directory) {
        var parts = filePath.split('/');
        parts.shift();
        parts.shift();
        var filename = parts[parts.length - 1];
        var extension = filename.split('.').pop().toUpperCase();
        if (extension == 'JSON' || extension == 'XML') {
            if (directory == 'datasources') {
                filename = parts.pop();
                filename = filename.replace(/\.[^/.]+$/, "");
                srcFiles[directory][filename] = './../../../' + parts.join('/');
            } else {
                srcFiles[directory].push('/../../' + parts.join('/'));
            }
        }
    }
    
    grunt.registerTask('reportTranslate', 'build translate to BIRT reports', function () {
		var languages = ['en_us', 'es_cl', 'es_co', 'es_es'];
		var srcWords = "assets/json/words/";
		var srcMessages = "assets/json/words/";
		var srcReport = "../../teknisa/est/reports/translationfiles/";
		var basePTBR = grunt.file.readJSON(srcMessages+"en_us.json");
        var languagePTBR = [];
        
        // regex to format properties file content
		var regexEmpty = /[\/\{\}\[\]\"]*/gm;
		var regexEqual = /\:/gm;
		var regexEnter = /\,/gm;
		var regexDoubleScape = /\\\\/gm;
		var regexScapedSpace = /u0020/gm;
		
        
        //Cria arquivo de tradução pt_br
        Object.keys(basePTBR).forEach( key => {
            languagePTBR.push({[unicodeEscape(key)]: unicodeEscape(key)});
        });
        
		var stringPTBRtoSave = JSON.stringify(languagePTBR).replace(regexEqual,'=').replace(regexEmpty,'').replace(regexEnter,'\n').replace(regexDoubleScape,'\\').replace(regexScapedSpace, ' ');
		grunt.file.write(srcReport+'translation'+".properties", stringPTBRtoSave);
		
		languages.forEach( lang => {
			var words = grunt.file.readJSON(srcWords+lang+".json");
			var properties = [];
            var filename = '';
			Object.keys(words).forEach(key => {
			    try{
				    properties.push({[unicodeEscape(key)]: unicodeEscape(words[key])});
			    }catch(err){
			        console.log(key);
			        throw err;
			    }
			});
			
			var stringToSave = JSON.stringify(properties).replace(regexEqual,'=').replace(regexEmpty,'').replace(regexEnter,'\n').replace(regexDoubleScape, '\\' ).replace(regexScapedSpace, ' ');
			filename = lang.split('_'); 
			filename = filename.shift()+'_'+filename.shift().toUpperCase(); 
			grunt.file.write(srcReport+'translation_'+filename+".properties", stringToSave); 
		});
	});

    grunt.registerTask('backendStep', 'build backend config files', function () {
        var srcFiles = {config: [], datasources: {}},
        directories = ['config', 'datasources'],
        dest = {
            config: '../backend/config/servicesMapping.json',
            datasources: '../backend/config/dataSourcesMapping.json'
        };

        srcFiles.config.push('/../../teknisa/est/config/ger.xml');

        directories.forEach(function (directory) {
            backendModules.forEach(function (module) {
                var moduleParts = module.split('/');
                var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
                var subModules = grunt.file.expand({
                    filter: 'isDirectory'
                }, module + '/src/Teknisa/' + moduleName + '/Modules/*');

                subModules.forEach(function (subModule) {
                    grunt.file.expand({
                        filter: 'isFile'
                    }, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
                        buildFileContent(srcFiles, filePath, directory);
                    });
                });

                grunt.file.expand({
                    filter: 'isFile'
                }, module + '/' + directory + '/**/*').forEach(function (filePath) {
                    buildFileContent(srcFiles, filePath, directory);
                });
            });
        });

        grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
        grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
    });

    grunt.loadNpmTasks('grunt-babel');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-copy');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-merge-json');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-zh-id-generator');

    grunt.registerTask('default', ['jshint', 'backendStep', 'concat', 'babel', 'zhWords', 'reportTranslate']);
    grunt.registerTask('dist', ['jshint', 'backendStep', 'uglify', 'cssmin']);
    grunt.registerTask('build', ['buildEnvironment:development', 'concat:buildLibs', 'buildAclStruct', 'copy', 'merge-json:messages']);
    
    grunt.loadNpmTasks('grunt-zh-words');
};